/*
Demonstrate various random distributions.
*/

#include "mathkit.hpp"

using namespace std;
using mathkit::Vector;

int main(void) {
	//set seed for random number generator
	mathkit::randseed();
	
	//poisson distribution
	cout << "Generating 100 random numbers of poisson distribution (lambda = 2)..." << endl;
	Vector poisdata = mathkit::rpois(2, 100);
	cout << "The sample mean is " << mathkit::mean(poisdata) << endl;
	cout << "The sample variance is " << mathkit::var(poisdata) << endl;
	cout << "The probability of X = 4 is " << mathkit::pois(2, 4) << endl;
	cout << "The probability of X <= 4 is " << mathkit::pois(2, 4, true) << endl;
	cout << endl;
	
	//binomial distribution
	cout << "Generating 100 random numbers of binomial distribution (n = 100, p = 0.5)..." << endl;
	Vector bindata = mathkit::rbinom(100, 0.5, 100);
	cout << "The sample mean is " << mathkit::mean(bindata) << endl;
	cout << "The sample variance is " << mathkit::var(bindata) << endl;
	cout << "The probability of X = 45 is " << mathkit::binom(100, 0.5, 45) << endl;
	cout << "The probability of X <= 45 is " << mathkit::binom(100, 0.5, 45, true) << endl;
	cout << endl;
	
	//exponential distribution
	cout << "Generating 100 random numbers of exponential distribution (theta = 2)..." << endl;
	Vector expdata = mathkit::rexp(2, 100);
	cout << "The sample mean is " << mathkit::mean(expdata) << endl;
	cout << "The sample variance is " << mathkit::var(expdata) << endl;
	cout << "The probability of X <= 1.5 is " << mathkit::expo(2, 1.5) << endl;
	cout << endl;
	
	//normal distribution
	cout << "Generating 100 random numbers of standard normal distribution..." << endl;
	Vector normdata = mathkit::rnorm(100);
	cout << "The sample mean is " << mathkit::mean(normdata) << endl;
	cout << "The sample variance is " << mathkit::var(normdata) << endl;
	cout << "The sample skewness is " << mathkit::skew(normdata) << endl;
	cout << "The sample kurtosis is " << mathkit::kurt(normdata) << endl;
	cout << "The probability of X <= 0.8 is " << mathkit::pnorm(0.8) << endl;
	cout << endl;
	
	//t distribution
	cout << "Cumulative probability and quantile of t distribution with 3 degrees of freedom." << endl;
	cout << "The probability of X <= 0.8 is " << mathkit::pt(0.8, 3) << endl;
	cout << "Quantile of p = 0.6 is x = " << mathkit::qt(0.6, 3) << endl;
	cout << endl;
	
	return 0;
}
