/*
Demonstrate the main functions for matrix and linear algebra.
*/

#include "Matrix.hpp"

using namespace std;
using namespace mathkit;

int main(void) {
	Matrix A(make_vec(9, 2., 1., 0., 1., 2., 1., 0., 1., 2.), Dimen(3, 3));
	cout << "A = " << to_str(A) << endl;
	cout << "number of rows: " << A.nrow() << endl;
	cout << "number of columns: " << A.ncol() << endl;
	if (A.square()) {
		cout << "A is a square matrix." << endl;
		cout << "The trace of A is " << trace(A) << endl;
	}
	cout << "A transpose is " << to_str(t(A)) << endl;
	if (A.symmetric()) {
		cout << "A is symmetric. So A transpose equals A." << endl;
		if (positive(A)) cout << "A is positive definite." << endl;
	}
	cout << "A(0, 1) = " << A(0, 1) << endl;
	cout << "Set A(0, 1) to 8..." << endl;
	A(0, 1) = 8;
	cout << "Now A = " << to_str(A) << endl;
	cout << endl;
	
	//determinant
	cout << "The determinant of A is " << det(A) << endl;
	cout << endl;
	
	//lu decomposition
	Matrices lup = lu(A);
	if (lup.empty()) cout << "Singular matrix!" << endl;
	else {
		Matrix &L = lup[0], &U = lup[1], &P = lup[2];
		cout << "L = " << to_str(L.round()) << endl;
		cout << "U = " << to_str(U.round()) << endl;
		cout << "P = " << to_str(P) << endl;
	}
	cout << endl;
	
	//qr decomposition
	Matrices qrmat = qr(A);
	if (qrmat.empty()) cout << "A has dependent columns!" << endl;
	else {
		Matrix &Q = qrmat[0], &R = qrmat[1];
		cout << "Q = " << to_str(Q.round()) << endl;
		cout << "R = " << to_str(R.round()) << endl;
	}
	cout << endl;
	
	//identity matrix
	Matrix I = eye(3);
	cout << "I = " << to_str(I) << endl;
	//multiplication and equality
	if (A * I == A) cout << "AI == A" << endl;
	cout << endl;
	
	//inverse matrix
	cout << "inv(A) = " << to_str(inv(A).round()) << endl;
	cout << endl;
	
	//ones and zeros matrix
	Matrix ones(Dimen(3, 3), 1), zeros(Dimen(3, 3));
	cout << "ones = " << to_str(ones) << endl;
	cout << "zeros = " << to_str(zeros) << endl;
	cout << endl;
	
	//diagonal elements and matrix
	Vector vec = diag(A);
	cout << "diag(A) = " << to_str(vec) << endl;
	cout << "diagonal matrix: " << to_str(diag(vec)) << endl;
	cout << endl;
	
	//rank
	Matrix B(make_vec(9, 1., 1., 1., 1., 2., 3., 2., 3., 4.), Dimen(3, 3));
	cout << "Singular matrix B = " << to_str(B) << endl;
	cout << "Rank of B is " << B.rank() << endl;
	cout << endl;
	
	//solve linear system Cx = b
	Matrix C(make_vec(4, 1., 3., 2., 4.), Dimen(2, 2));
	Vector b = make_vec(2, 5., 6.);
	Vector result = linsolve(C, b);
	cout << "C = " << to_str(C) << endl;
	cout << "b = " << to_str(b) << endl;
	cout << "solution of Cx = b is " << to_str(result) << endl;
	
	return 0;
}
