/*
Calculate the yield to maturity of a bond with coupon rate 8%, face value 1000 and 5 years to maturity.
Given that the current market price of this bond is 1105.
*/

#include <iostream>
#include "mathkit.hpp"

using namespace std;
using mathkit::Pair;

double equation(double rate) {
	return mathkit::apv(80, rate, 5) + mathkit::pv(1000, rate, 5) - 1105;
}

int main(void) {
	Pair guess(0, 0.2);
	if (mathkit::probe(equation, guess)) {
		double ytm = mathkit::solve(equation, guess);
		cout.precision(3);
		cout << "The yield to maturity is " << ytm * 100 << "%";
	}
	else {
		cout << "Probe for maturity yield failed.";
	}
	cout << endl;
	return 0;
}
