/*
Save the input and output values of an arbitrary function to file.
Then the saved data file could be used to plot a graph of that function.
Many plotting softwares could do this job well like gnuplot etc.
And you could also load the data from file later.
*/

#include <fstream>
#include "mathkit.hpp"

using namespace std;
using mathkit::Vector;
using mathkit::Table;

double equation(double i) {
	return mathkit::apv(80, i, 5) + mathkit::pv(1000, i, 5) - 1105;
}

int main(void) {
	Vector xvec = mathkit::linspace(0, 0.1);
	Vector yvec = mathkit::each(equation, xvec);
	
	Table data;
	data.push_back(xvec);
	data.push_back(yvec);
	
	ofstream ofs("savedata.txt");
	mathkit::save(ofs, data);
	ofs.close();
	cout << "Data saved to file savedata.txt." << endl;
	
	data.clear();
	
	ifstream ifs("savedata.txt");
	data = mathkit::load(ifs, 100, 2);
	ifs.close();
	cout << "Data loaded from file savedata.txt." << endl;
	cout << "number of rows: " << data[0].size() << endl;
	cout << "number of columns: " << data.size() << endl;
	
	return 0;
}
