mathkit <- list()

mathkit$pv <- function(payment, rate, period) return(payment / (1 + rate) ^ period)
mathkit$fv <- function(payment, rate, period) return(payment * (1 + rate) ^ period)
mathkit$pv_coef <- function(rate, period) return((1 + rate) ^ -period)
mathkit$fv_coef <- function(rate, period) return((1 + rate) ^ period)

mathkit$apv_coef <- function(rate, period, prepaid = FALSE) {
  if (rate == 0) return(period)
  coef <- (1 - (1 + rate) ^ -period) / rate
  if (prepaid) coef <- coef * (1 + rate)
  return(coef)
}

mathkit$afv_coef <- function(rate, period, prepaid = FALSE) {
  if (rate == 0) return(period)
  coef <- ((1 + rate) ^ period - 1) / rate
  if (prepaid) coef <- coef * (1 + rate)
  return(coef)
}

mathkit$apv <- function(annuity, rate, period, prepaid = FALSE) return(annuity * mathkit$apv_coef(rate, period, prepaid))
mathkit$afv <- function(annuity, rate, period, prepaid = FALSE) return(annuity * mathkit$afv_coef(rate, period, prepaid))

mathkit$spv <- function(payments, rate, prepaid = FALSE) {
  periods <- 1:length(payments)
  if (prepaid) periods <- periods - 1
  return(sum(payments * sapply(periods, function(n) return(mathkit$pv_coef(rate, n)))))
}

mathkit$sfv <- function(payments, rate, prepaid = FALSE) {
  periods <- (length(payments) - 1):0
  if (prepaid) periods <- periods + 1
  return(sum(payments * sapply(periods, function(n) return(mathkit$fv_coef(rate, n)))))
}

mathkit$comp_rate <- function(pval, fval, period) return((fval / pval) ^ (1 / period) - 1)

mathkit$linregress <- function(x, y) {
  b = cov(x, y) / var(x)
  a = mean(y) - b * mean(x)
  return(list(slope = b, intercept = a))
}

mathkit$dot_prod <- function(x, y) return(sum(x * y))

mathkit$cross_prod <- function(x, y) {
  v <- vector(length = 3)
  v[1] <- x[2] * y[3] - x[3] * y[2]
  v[2] <- x[3] * y[1] - x[1] * y[3]
  v[3] <- x[1] * y[2] - x[2] * y[1]
  return(v)
}

mathkit$perm <- function(m, n) return(prod(m:(m - n + 1)))
mathkit$comb <- function(m, n) return(mathkit$perm(m, n) / factorial(n))

mathkit$gcd <- function(a, b) {
  if (b == 0) return(a)
  return(mathkit$gcd(b, a %% b))
}

mathkit$lcm <- function(a, b) return(a * b / mathkit$gcd(a, b))

mathkit$probe <- function(func, guess, factor = 1.6, ntry = 50) {
  lv <- func(guess[1])
  rv <- func(guess[2])
  while (ntry > 0) {
    if (lv * rv <= 0) return(list(success = TRUE, region = guess))
    if (abs(lv) < abs(rv)) {
      guess[1] <- guess[1] - factor * (guess[2] - guess[1])
      lv <- func(guess[1])
    }
    else {
      guess[2] <- guess[2] + factor * (guess[2] - guess[1])
      rv <- func(guess[2])
    }
    ntry <- ntry - 1
  }
  return(list(success = FALSE, region = guess))
}

mathkit$scan <- function(func, scope, n) {
  marks <- seq(scope[1], scope[2], length = n + 1)
  regions <- list()
  idx <- 1
  for (i in 1:n) 
    if (func(marks[i]) * func(marks[i+1]) <= 0) {
      regions[[idx]] <- c(marks[i], marks[i+1])
      idx <- idx + 1
    }
  return(regions)
}

mathkit$solve <- function(func, region, eps = 0.000001) {
  if (func(region[1]) == 0) return(region[1])
  if (func(region[2]) == 0) return(region[2])
  repeat {
    mid <- (region[1] + region[2]) / 2
    if (region[2] - region[1] < eps || func(mid) == 0) return(mid)
    if (func(region[1]) * func(mid) < 0) {region[2] <- mid}
    else {region[1] <- mid}
  }
}
